% generate_spike_trains.m
%
% Anthony Kellems, 6/11/08
%
% Generates N Poisson spike trains with rate lambda Hz for Tmax seconds
% from a homogeneous Poisson process.

function train = generate_spike_trains(N,lambda,Tmax)

s = zeros(ceil(1.003*Tmax*lambda*N),1);     % preallocate approximate # of spikes
                                     % The 1.01 is a factor to overestimate
                                     % spikes a bit, which gives a big
                                     % speedup in case we go over the
                                     % estimate Tmax*lambda*N. Change this
                                     % factor as you see fit.
ct = 1;

s(ct) = -log(rand)/lambda;      % first spike

while s(ct) < Tmax*N,           % loop for N*Tmax (will scale to Tmax later)
    snew = -log(rand)/lambda;
    s(ct+1) = s(ct) + snew;     % Poisson process: generate spikes
    ct = ct+1;
end

s(ct) = (Tmax + 10)*N;
s(ct+1:end) = [];                % remove unused preallocated space
                                    %leave in one spike after Tmax for
                                    %later coding purposes

spikes = length(s);            % total number of spikes

s = s/N;                        % scale back to desired time interval

neuronVec = ceil(rand(spikes,1)*N);     % get neurons for each spike

train = [s neuronVec];          % complete spike train matrix